==> ui-bs-button-choice-0.0.9.tgz: package/addon/components/ui-bs-button-choice.js <==
import Ember from 'ember';
import layout from '../templates/components/ui-bs-button-choice';

var isEmpty = Ember.isEmpty;
var typeOf = Ember.typeOf;

export default Ember.Component.extend({
	// Component Properties
  layout: layout,
	tagName: 'div',
	classNames: ['ui-bs-button-choice','btn-group'],
	classNameBindings: ['stretch'],
	attributeBindings: ['dataToggle:data-toggle'],
	dataToggle: 'buttons',
	// Data Properties
	choices: null,
	labelProp: 'name', // the property name to display when given a hash
	valueProp: 'id', // the property name to use as a reference key when given a hash
	type: 'radio', // radio/checkbox
	missingChoicesText: 'no choices available',
	canBeNull: null,
	_canBeNull: Ember.on('didInsertElement', function() {
		Ember.run.next(this, function() {
			if(this.get('canBeNull') === null) {
				this.set('canBeNull', this.get('type') === 'radio' ? false : true);
			}
		});
	}),
	color: 'default', 
	colorClass: Ember.computed('color', function() {
		return 'btn-' + this.get('color');
	}),
	size: 'default',
	sizeClass: Ember.computed('size', function() {
		var size = this.get('size');
		if(Ember.A(['lg','large']).contains(size)) {
			return 'btn-lg';
		} else if(Ember.A(['sm','small']).contains(size)) {
			return 'btn-sm';
		} else if(Ember.A(['xs','tiny']).contains(size)) {
			return 'btn-xs';
		} else {
			return false;
		}
	}),
	stretch: false, // stretch the buttons to fit the available width
	_stretchObserver: Ember.observer('stretch','choicesHash','_componentWidth', function() {
		let stretch = this.get('stretch');
		let width = this.get('_componentWidth');
		let buttons = this.get('choicesHash').length;
		if(stretch && buttons > 0) {
      let widthStyle = String( Math.floor(Number(width/buttons)));
			this.set('_widthStyle', Ember.String.htmlSafe(`width: ${widthStyle}px`));
		}
	}),
  _stretchInit: Ember.on('didInsertElement', function() {
    this._stretchObserver();
  }),
	_componentWidth: null,
	_componentWidthObserver: Ember.observer('resizeDidHappen', function() {
		this.set('_componentWidth',   this.$().get(0).innerWidth || this.$().get(0).offsetWidth);			
	}),
  _componentInit: Ember.on('didInsertElement', function() {
    this._componentWidthObserver();
  }),
	resizeEventListener: Ember.observer('stretch', function() {
		if(this.get('stretch')) {
			var localisedResize = 'resize.' + this.get('elementId');
      this.$(window).on(localisedResize, Ember.run.bind(this, this.trigger, ['resizeDidHappen']));
		}
	}),
  resizeEventListenerInit: Ember.on('didInsertElement', function() {
    this.resizeEventListener();
  }),
	resizeEventCleanup: Ember.on('willDestroyElement', function() {
			var localisedResize = 'resize.' + this.get('elementId');
			this.$(window).off(localisedResize);
	}),
	selected: null,
	selectedObject: null,
	choicesHash: Ember.A([]),
	/**
		Set the `choicesHash` to the appropriate value. Choices will come in as one of the following formats:
	
		1) An array of objects (the easiest to deal with)
		2) An array of literals (numbers or strings)
		3) A comma seperated string (which is just converted to #1)
		4) A deferred array of objects such as a Promise or specifically an Ember-Data Promise
		5) A deferred object which has an offset/deep property of which contains the array
	
		In all cases the aim is to set "choicesHash" to format #1
	*/
	_choicesHash: Ember.observer('choices.length','selected', function() {
		Ember.run.next( () => {
			var self = this;
			var p = this.getProperties('choices','labelProp','valueProp','type','selected');
			if(typeOf(p.choices) === 'string') {
				p.choices = p.choices.split(',');
			}
			if(isEmpty(p.choices)) {
				this.set('choicesHash', Ember.A([]));
				return;
			}
			// is it an Ember promise?
			if(Ember.A(['class','instance']).contains(typeOf(p.choices)) ) {
				if(p.choices.isFulfilled) {
					p.choices = p.choices.get('content');
					p.choices = this.processPromise(p.choices);					
				} else {
					p.choices.then(function(promisedChoices) {
						promisedChoices = self.processPromise(promisedChoices);
						self.processChoiceHash(promisedChoices, p.selected, {valueProp: p.valueProp, labelProp: p.labelProp});
					});
				}
			}
			this.set('choicesHash', this.processChoiceHash(p.choices, p.selected, {valueProp: p.valueProp, labelProp: p.labelProp}) );				
		});
	}),
  _choicesHashInit: Ember.on('didInsertElement', function() {
    this._choicesHash();
  }),
	processChoiceHash: function(choices,selected,options) {
    choices = Ember.A(choices || []);
		if(typeOf(choices) === 'array') {
			var choicesHash = Ember.A(choices.map(function(item) {
				var hash; 
				if(Ember.A(['string','number']).contains(typeOf(item))) {
					hash = {};
					hash.id = String(item);
					hash.name = String(item);
				} else {
					// copy object
					hash = JSON.parse(JSON.stringify(item));
					// ensure interface is met
					hash.id = String(item[options.valueProp]);
					hash.name = String(item[options.labelProp]);
				}
				// Selected?
				if (
					(typeOf(selected) === 'array' && selected.contains(hash.id)) ||
					(typeOf(selected) === 'string' && selected === hash.id)
				) {
					hash.checked = true;
				}
				
				return hash;
			}));
			if(isEmpty(choicesHash.filterBy('checked',true)) && !this.get('canBeNull')) {
				choicesHash[0].checked = true;
				this.set('selected',choicesHash[0].id);
			}
			
			return Ember.A(choicesHash);
		} else {
			return Ember.A([]);
		}
	},
	processPromise: function(promise) {
		if(promise.id) {
			var choicesOffset = this.get('choicesOffset');
			if(choicesOffset) {
				// an explicit statement about what property to use is available
				promise = promise._data[choicesOffset];
			} else {
				console.warn('%s retrieved a promise object but no offset property was provided to populate the choices.', this.get('elementId'));
				promise = [];
			}
		} else {
			promise = promise.get('content').map(function(item) {
				return item._data;
			});
		}
		return promise;
	},
	click: function() {
		Ember.run.next( ()=> {
			var action = this.processEvent();
			if(action.hasChanged) {
				this.set('selected', action.selected);
				this.set('selectedObject', action.selectedObject);
				this.sendAction('changeEvent', action);
			} else if(this.get('canBeNull')) {
				this.set('selected', null);
				this.set('selectedObject', null);
			}
		});
	},
	processEvent: function() {
		var selected;
		var selectedObject;
		var self = this;
		if (this.get('type') === 'radio') {
			selected = this.$('.active input').val();
			selectedObject = this.get('choicesHash').findBy('id',selected);
		} else {
			selected = Ember.A([]);
			selectedObject = Ember.A([]);
			this.$('.active input').each( ()=> {
        var newItem = $(this).val();
        // let newItem = this.$().val();
				selected.pushObject(newItem);
				selectedObject.pushObject(self.get('choicesHash').findBy('id',newItem));
			});
		}
    
		// determine the change (if any)
		var componentState = this.getProperties('selected','selectedObject');
		var type = this.get('type');
		var state = {
			isEmpty: null,
			howMany: null,
			hasChanged: null, // maybe be overridden below
			changeSource: null, // [external,bootstrap]
			added: Ember.A([]), // list of ID(s) just added 
			removed: Ember.A([]) // list of ID(s) just removed 
		};
		// radio versus checkbox
		if(type === 'radio') {
			// existance
			if(isEmpty(selected)) {
				state.isEmpty = true;
				state.howMany = 0;
			} else {
				state.isEmpty = false;
				state.howMany = 1;
			}
			// changes
			if(componentState.selected !== selected ) {
				state.hasChanged = true;
				state.changeSource = 'bootstrap';
				if(!state.isEmpty) {
					state.added = [ selected ];
				}
				if(!Ember.isEmpty(componentState.selected)) {
					state.removed = [ componentState.selected ];
				}
			} else {
				// if change was done directly to "selected" bound property
				// we will end up in this state but there will be delta in selectedObject
				if(isEmpty(componentState.selectedObject) || componentState.selectedObject.id !== selectedObject.id) {
					state.hasChanged = true;
					state.changeSource = 'external';	
					if(!isEmpty(selectedObject)) {
						state.added = [ selectedObject.id ];						
					}
					if(!isEmpty(componentState.selectedObject)) {
						state.removed = [ componentState.selectedObject.id ];						
					}					
				} else {
					state.hasChanged = false;
				}
			}
      
		} // end Radio 
    else {
			if(Ember.isEmpty(selected)) {
				state.isEmpty = true;
				state.howMany = 0;
			} else {
				state.isEmpty = false;
				state.howMany = selected.length;
			}
			// changes
			componentState.selected = isEmpty(componentState.selected) ? [] : componentState.selected;
			if(typeOf(componentState.selected) === 'string') {
				// a checkbox's should always be a selector but if its a string then convert gracefully and warn to console
				componentState.selected = componentState.selected.split(',');
			}
			selected = selected || Ember.A([]);
			if(componentState.selected.join(',') !== selected.join(',')) {
				state.hasChanged = true;
				state.changeSource = 'bootstrap';
				componentState.selected.forEach(function(item) {
					if (!Ember.A(selected).contains(item)) {
						state.removed.addObject(item);
					}
				});
				selected.forEach(function(item) {
					if (!Ember.A(componentState.selected).contains(item)) {
						state.added.addObject(item);
					}
				});
			} else {
				// if change was done directly to "selected" bound property
				// we will end up in this state but there will be delta in selectedObject
				componentState.selectedObject = componentState.selectedObject || Ember.A([]);
				if(Ember.A(componentState.selectedObject).mapBy('id').join(',') !== selectedObject.mapBy('id').join(',')) {
					state.hasChanged = true;
					state.changeSource = 'external';	
					if(!isEmpty(selectedObject)) {
						selectedObject.mapBy('id').forEach(function(item) {
							if(!Ember.A(componentState.selectedObject.mapBy('id')).contains(item)) {
								state.added.addObject(item);
							}
						});				
						Ember.A(componentState.selectedObject).mapBy('id').forEach(function(item) {
							if(!selectedObject.mapBy('id').contains(item)) {
								state.removed.addObject(item);
							}
						});				
					}
					if(!isEmpty(componentState.selectedObject)) {
						state.removed = [ componentState.selectedObject.id ];						
					}					
				} else {
					state.hasChanged = false;
				}
			}
		}
		// handle no selections if not allowed
		if(!this.get('canBeNull') && state.isEmpty) {
			// TODO: implement
			console.log('property that can not be null, IS.');
		}
		state.selected = selected;
		state.selectedObject = selectedObject;
		return state;
	},
	selectedObserver: Ember.observer('selected', function() {
		Ember.run.next(this, function() {
			var action = this.processEvent();
			if(action.hasChanged) {
				this.set('selectedObject', action.selectedObject);
			} 
		});
	})
});

==> ui-bs-button-choice-0.0.10.tgz: package/addon/components/ui-bs-button-choice.js <==
import Ember from 'ember';
import layout from '../templates/components/ui-bs-button-choice';

var isEmpty = Ember.isEmpty;
var typeOf = Ember.typeOf;

export default Ember.Component.extend({
	// Component Properties
  layout: layout,
	tagName: 'div',
	classNames: ['ui-bs-button-choice','btn-group'],
	classNameBindings: ['stretch'],
	attributeBindings: ['dataToggle:data-toggle'],
	dataToggle: 'buttons',
	// Data Properties
	choices: null,
	labelProp: 'name', // the property name to display when given a hash
	valueProp: 'id', // the property name to use as a reference key when given a hash
	type: 'radio', // radio/checkbox
	missingChoicesText: 'no choices available',
	canBeNull: null,
	_canBeNull: Ember.on('didInsertElement', function() {
		Ember.run.next(this, function() {
			if(this.get('canBeNull') === null) {
				this.set('canBeNull', this.get('type') === 'radio' ? false : true);
			}
		});
	}),
	color: 'default', 
	colorClass: Ember.computed('color', function() {
		return 'btn-' + this.get('color');
	}),
	size: 'default',
	sizeClass: Ember.computed('size', function() {
		var size = this.get('size');
		if(Ember.A(['lg','large']).contains(size)) {
			return 'btn-lg';
		} else if(Ember.A(['sm','small']).contains(size)) {
			return 'btn-sm';
		} else if(Ember.A(['xs','tiny']).contains(size)) {
			return 'btn-xs';
		} else {
			return false;
		}
	}),
	stretch: false, // stretch the buttons to fit the available width
	_stretchObserver: Ember.observer('stretch','choicesHash','_componentWidth', function() {
		let stretch = this.get('stretch');
		let width = this.get('_componentWidth');
		let buttons = this.get('choicesHash').length;
		if(stretch && buttons > 0) {
      let widthStyle = String( Math.floor(Number(width/buttons)));
			this.set('_widthStyle', Ember.String.htmlSafe(`width: ${widthStyle}px`));
		}
	}),
  _stretchInit: Ember.on('didInsertElement', function() {
    this._stretchObserver();
  }),
	_componentWidth: null,
	_componentWidthObserver: Ember.observer('resizeDidHappen', function() {
		this.set('_componentWidth',   this.$().get(0).innerWidth || this.$().get(0).offsetWidth);			
	}),
  _componentInit: Ember.on('didInsertElement', function() {
    this._componentWidthObserver();
  }),
	resizeEventListener: Ember.observer('stretch', function() {
		if(this.get('stretch')) {
			var localisedResize = 'resize.' + this.get('elementId');
      this.$(window).on(localisedResize, Ember.run.bind(this, this.trigger, ['resizeDidHappen']));
		}
	}),
  resizeEventListenerInit: Ember.on('didInsertElement', function() {
    this.resizeEventListener();
  }),
	resizeEventCleanup: Ember.on('willDestroyElement', function() {
			var localisedResize = 'resize.' + this.get('elementId');
			this.$(window).off(localisedResize);
	}),
	selected: null,
	selectedObject: null,
	choicesHash: Ember.A([]),
	/**
		Set the `choicesHash` to the appropriate value. Choices will come in as one of the following formats:
	
		1) An array of objects (the easiest to deal with)
		2) An array of literals (numbers or strings)
		3) A comma seperated string (which is just converted to #1)
		4) A deferred array of objects such as a Promise or specifically an Ember-Data Promise
		5) A deferred object which has an offset/deep property of which contains the array
	
		In all cases the aim is to set "choicesHash" to format #1
	*/
	_choicesHash: Ember.observer('choices.length','selected', function() {
		Ember.run.next( () => {
			var self = this;
			var p = this.getProperties('choices','labelProp','valueProp','type','selected');
			if(typeOf(p.choices) === 'string') {
				p.choices = p.choices.split(',');
			}
			if(isEmpty(p.choices)) {
				this.set('choicesHash', Ember.A([]));
				return;
			}
			// is it an Ember promise?
			if(Ember.A(['class','instance']).contains(typeOf(p.choices)) ) {
				if(p.choices.isFulfilled) {
					p.choices = p.choices.get('content');
					p.choices = this.processPromise(p.choices);					
				} else {
					p.choices.then(function(promisedChoices) {
						promisedChoices = self.processPromise(promisedChoices);
						self.processChoiceHash(promisedChoices, p.selected, {valueProp: p.valueProp, labelProp: p.labelProp});
					});
				}
			}
			this.set('choicesHash', this.processChoiceHash(p.choices, p.selected, {valueProp: p.valueProp, labelProp: p.labelProp}) );				
		});
	}),
  _choicesHashInit: Ember.on('didInsertElement', function() {
    this._choicesHash();
  }),
	processChoiceHash: function(choices,selected,options) {
    choices = Ember.A(choices || []);
		if(typeOf(choices) === 'array') {
			var choicesHash = Ember.A(choices.map(function(item) {
				var hash; 
				if(Ember.A(['string','number']).contains(typeOf(item))) {
					hash = {};
					hash.id = String(item);
					hash.name = String(item);
				} else {
					// copy object
					hash = JSON.parse(JSON.stringify(item));
					// ensure interface is met
					hash.id = String(item[options.valueProp]);
					hash.name = String(item[options.labelProp]);
				}
				// Selected?
				if (
					(typeOf(selected) === 'array' && selected.contains(hash.id)) ||
					(typeOf(selected) === 'string' && selected === hash.id)
				) {
					hash.checked = true;
				}
				
				return hash;
			}));
			if(isEmpty(choicesHash.filterBy('checked',true)) && !this.get('canBeNull')) {
				choicesHash[0].checked = true;
				this.set('selected',choicesHash[0].id);
			}
			
			return Ember.A(choicesHash);
		} else {
			return Ember.A([]);
		}
	},
	processPromise: function(promise) {
		if(promise.id) {
			var choicesOffset = this.get('choicesOffset');
			if(choicesOffset) {
				// an explicit statement about what property to use is available
				promise = promise._data[choicesOffset];
			} else {
				console.warn('%s retrieved a promise object but no offset property was provided to populate the choices.', this.get('elementId'));
				promise = [];
			}
		} else {
			promise = promise.get('content').map(function(item) {
				return item._data;
			});
		}
		return promise;
	},
	click: function() {
		Ember.run.next( ()=> {
			var action = this.processEvent();
			if(action.hasChanged) {
				this.set('selected', action.selected);
				this.set('selectedObject', action.selectedObject);
				this.sendAction('changeEvent', action);
			} else if(this.get('canBeNull')) {
				this.set('selected', null);
				this.set('selectedObject', null);
			}
		});
	},
	processEvent: function() {
		var selected;
		var selectedObject;
		var self = this;
		if (this.get('type') === 'radio') {
			selected = this.$('.active input').val();
			selectedObject = this.get('choicesHash').findBy('id',selected);
		} else {
			selected = Ember.A([]);
			selectedObject = Ember.A([]);
			this.$('.active input').each( ()=> {
        // var newItem = $(this).val();
        let newItem = this.$().val();
				selected.pushObject(newItem);
				selectedObject.pushObject(self.get('choicesHash').findBy('id',newItem));
			});
		}
    
		// determine the change (if any)
		var componentState = this.getProperties('selected','selectedObject');
		var type = this.get('type');
		var state = {
			isEmpty: null,
			howMany: null,
			hasChanged: null, // maybe be overridden below
			changeSource: null, // [external,bootstrap]
			added: Ember.A([]), // list of ID(s) just added 
			removed: Ember.A([]) // list of ID(s) just removed 
		};
		// radio versus checkbox
		if(type === 'radio') {
			// existance
			if(isEmpty(selected)) {
				state.isEmpty = true;
				state.howMany = 0;
			} else {
				state.isEmpty = false;
				state.howMany = 1;
			}
			// changes
			if(componentState.selected !== selected ) {
				state.hasChanged = true;
				state.changeSource = 'bootstrap';
				if(!state.isEmpty) {
					state.added = [ selected ];
				}
				if(!Ember.isEmpty(componentState.selected)) {
					state.removed = [ componentState.selected ];
				}
			} else {
				// if change was done directly to "selected" bound property
				// we will end up in this state but there will be delta in selectedObject
				if(isEmpty(componentState.selectedObject) || componentState.selectedObject.id !== selectedObject.id) {
					state.hasChanged = true;
					state.changeSource = 'external';	
					if(!isEmpty(selectedObject)) {
						state.added = [ selectedObject.id ];						
					}
					if(!isEmpty(componentState.selectedObject)) {
						state.removed = [ componentState.selectedObject.id ];						
					}					
				} else {
					state.hasChanged = false;
				}
			}
      
		} // end Radio 
    else {
			if(Ember.isEmpty(selected)) {
				state.isEmpty = true;
				state.howMany = 0;
			} else {
				state.isEmpty = false;
				state.howMany = selected.length;
			}
			// changes
			componentState.selected = isEmpty(componentState.selected) ? [] : componentState.selected;
			if(typeOf(componentState.selected) === 'string') {
				// a checkbox's should always be a selector but if its a string then convert gracefully and warn to console
				componentState.selected = componentState.selected.split(',');
			}
			selected = selected || Ember.A([]);
			if(componentState.selected.join(',') !== selected.join(',')) {
				state.hasChanged = true;
				state.changeSource = 'bootstrap';
				componentState.selected.forEach(function(item) {
					if (!Ember.A(selected).contains(item)) {
						state.removed.addObject(item);
					}
				});
				selected.forEach(function(item) {
					if (!Ember.A(componentState.selected).contains(item)) {
						state.added.addObject(item);
					}
				});
			} else {
				// if change was done directly to "selected" bound property
				// we will end up in this state but there will be delta in selectedObject
				componentState.selectedObject = componentState.selectedObject || Ember.A([]);
				if(Ember.A(componentState.selectedObject).mapBy('id').join(',') !== selectedObject.mapBy('id').join(',')) {
					state.hasChanged = true;
					state.changeSource = 'external';	
					if(!isEmpty(selectedObject)) {
						selectedObject.mapBy('id').forEach(function(item) {
							if(!Ember.A(componentState.selectedObject.mapBy('id')).contains(item)) {
								state.added.addObject(item);
							}
						});				
						Ember.A(componentState.selectedObject).mapBy('id').forEach(function(item) {
							if(!selectedObject.mapBy('id').contains(item)) {
								state.removed.addObject(item);
							}
						});				
					}
					if(!isEmpty(componentState.selectedObject)) {
						state.removed = [ componentState.selectedObject.id ];						
					}					
				} else {
					state.hasChanged = false;
				}
			}
		}
		// handle no selections if not allowed
		if(!this.get('canBeNull') && state.isEmpty) {
			// TODO: implement
			console.log('property that can not be null, IS.');
		}
		state.selected = selected;
		state.selectedObject = selectedObject;
		return state;
	},
	selectedObserver: Ember.observer('selected', function() {
		Ember.run.next(this, function() {
			var action = this.processEvent();
			if(action.hasChanged) {
				this.set('selectedObject', action.selectedObject);
			} 
		});
	})
});
